from musurgia.pdf.pdf import Pdf
from musurgia.unittest import TestCase


class TestRowContainer(TestCase):
    def setUp(self) -> None:
        self.pdf = Pdf()

    def test_get_relative_x2(self):
        self.assertFalse(True)

    def test_get_relative_y2(self):
        self.assertFalse(True)

    def test_get_height(self):
        self.assertFalse(True)

    def test_get_width(self):
        self.assertFalse(True)
